/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include <kio/slaveconfig.h>
#include <kio/scheduler.h>
#include <kio/global.h>
#include <kmimetype.h>
#include <dcopclient.h>
#include "taviahtmlview.h"
#include "taviahtmlpopup.h"
#include <qtimer.h>
#include <kconfig.h>
#include <qbrush.h>
#include <qpen.h>
#include <qfile.h>


TaviaHTMLView::TaviaHTMLView(KURL url, KParts::URLArgs args, QWidget *parent, const char *name, WFlags f)
 :KHTMLPart(parent)
{
    //QTimer::singleShot(0,Cache::loader(),SLOT(setSlaveConfig()));
    //KIO::Scheduler::publishSlaveOnHold();
    //KIO::SlaveConfig::self()->setConfigData("http","","UserAgent","Tavia-Test");

    // Ȥꤢʤ󤫽񤤤Ȥ
    //begin("");
    //write("<html />");
    //end();

    completed = true;
    locked = false;
    backAction = false;
    forwardAction = false;
    reloadAction = false;
    stopAction = false;
    noCache = false;
    run = 0;
    urlHistory.setAutoDelete(true);
    emit sendActionStatus();
    kdDebug() << "GUI:" << xmlFile() << endl;
    //setXMLFile("khtml_browser.rc",true,true);
    setXMLFile(locate("data","tavia/taviahtmlviewui.rc"),true,true);

    currentHistory = 0;
    noHistoryUpdate = false;
    reloadHistory = false;
    connect(this,SIGNAL(completed()),SLOT(slotCompleted()));
    connect(this,SIGNAL(completed(bool)),SLOT(slotCompleted(bool)));
    connect(browserExtension(),SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
        this,SLOT(slotOpenURL(const KURL &,const KParts::URLArgs &)));
    connect(this,SIGNAL(setStatusBarText(const QString&)),
        this,SLOT(slotSetStatusBarText(const QString&)));
    connect(this,SIGNAL(popupMenu(const QString &,const QPoint &)),SLOT(popupMenu(const QString &)));
    //connect(browserExtension(),SIGNAL(openURLRequestDelayed(const KURL &,const KParts::URLArgs &)),
    //    this,SLOT(tSetURLArgs(const KURL &,const KParts::URLArgs &)));
    connect(partManager(),SIGNAL(partAdded(KParts::Part *)),this,SLOT(partAdded(KParts::Part *)));
    connect(partManager(),SIGNAL(partRemoved(KParts::Part *)),this,SLOT(partRemoved(KParts::Part *)));
    setJScriptEnabled(true);
    setJavaEnabled(false);
    setPluginsEnabled(false);
    setMetaRefreshEnabled(false);
    //setAutoloadImages(false);
    setDNDEnabled(true);

    popup = new TaviaHTMLPopup(this,"popup");

    KConfig* config = KGlobal::config();
    config->setGroup("thumbnail");
    thumbnailUpdateTimer = config->readNumEntry("updateInterval");
    //if (thumbnailUpdateTimer != 0)
    //    connect(this,SIGNAL(docCreated()),this,SLOT(createThumbnail()));
    connect(browserExtension(),SIGNAL(loadingProgress(int)),this,SLOT(progress(int)));

    fileIO = new TaviaFileIO(this);

    if (!url.isEmpty()) {
        pURL = url;
        pArgs = args;
        browserExtension()->openURLRequest(pURL,pArgs);
    }
}


TaviaHTMLView::~TaviaHTMLView()
{
    if (run != 0) delete run;
    delete popup;
}

/*!
    \fn TaviaHTMLView::slotOpenURL(const KURL &url,const KParts::URLArgs &args)
    URLºݤ˥ץ󤵤륹åȡ
 */
void TaviaHTMLView::slotOpenURL(const KURL &url,const KParts::URLArgs &args)
{
    slotOpenURL(url,args,0);
}

/*!
    \fn TaviaHTMLView::slotOpenURL(const KURL &url,const KParts::URLArgs &args,KHTMLPart *part)
    URLºݤ˥ץ󤵤륹åȡ
 */
void TaviaHTMLView::slotOpenURL(const KURL &url,const KParts::URLArgs &args,KHTMLPart *part)
{
    pURL = url;
    kdDebug() << "slotOpenURL : " << pURL.url() << endl;
    if (pURL.isEmpty()) return;
    pArgs = args;
    pProgress = 0;
    emit startOpenURL();
    emit pageURLReady(this,pURL);
    noHistoryUpdate = false;
    if (pURL.url().find(QString::fromLatin1("taviainfo:"),0,false ) == 0) {
        pURL = pURL.url().right( pURL.url().length() - 10);
    }
    // ime.nuХѥ
    // referrerϤΤޤ޻Ĥޤ
    KConfig* config = KGlobal::config();
    config->setGroup("URLfilter");
    if (config->readBoolEntry("imenuBypass",false)) {
        if (pURL.url().find(QString::fromLatin1("http://ime.nu"),0,false ) == 0) {
            if (pArgs.metaData().contains("referrer")) {
                pArgs.metaData().replace("referrer",pURL.url());
            }
            pURL = "http:/" + pURL.url().right( pURL.url().length() - 13);
        }
    }

    // "javascript"ǤϤޤURLϥץȥexecuteScript
    // Ǥ٤Ȥʤġ
    if (pURL.url().find(QString::fromLatin1("javascript:"),0,false ) == 0) {
        executeScript(KURL::decode_string(pURL.url().right( pURL.url().length() - 11)));
        return;
    }

    if (pURL.url() == "about:gpl") {
        //noCache = true;
        aboutGPL();
        return;
    } else if (pURL.url().find(QString::fromLatin1("about:"),0,false) == 0) {
        aboutTavia();
        return;
    }
    // MimeTypeå¸Ū
    // ΥɤϳĥҤˤå
    /*
    KMimeType::Ptr m = KMimeType::findByURL(url);
    kdDebug() << "KMimeType: " << m->name() << endl;
    if (m->name().lower().startsWith("image")) {
        noCache = true;
        outputImage(url);
        return;
    }
    */

    //tSetURLArgs(url,args);
    // ʲ target="_blank" Ф
    // פȻפΤ
    // ˼٤Ǥʤȹͤ롣Τ
    if ((args.frameName.lower() == QString::fromLatin1("_blank")) ||
        locked) {
        KParts::URLArgs newargs = pArgs;
        newargs.frameName = "";
        emit browserExtension()->createNewWindow(url,newargs);
        return;
    }
    browserExtension()->setURLArgs(pArgs);
    //tSetURLArgs(url,args);
    stopAction = true;
    reloadAction = false;
    emit sendActionStatus(pURL);
    completed = false;
    //openURL(url);
    //jobnum = fileIO->mimetype(url,args);
    //connect(fileIO,SIGNAL(sendMimeType(int, const QString)),
    //    this,SLOT(slotMimeType(int, const QString)));
    if (pURL.protocol() == "" || pURL.isLocalFile()) {
        KURL u;
        u.setProtocol("file");
        if (pURL.path().at(0) != '/') {
            if (pURL.protocol() == "") u.setPath("/" + pURL.url());
            else u.setPath("/" + pURL.path());
        } else u.setPath(pURL.path());
        pURL = u;
        // ϤʲפǤ롣
        //if (!QFile::exists(pURL.path())) {
        //    // ⤦ޥˡͤʤȡ
        //    QString d;
        //    d = "<html>\n<head><title>";
        //    d += i18n("No such file or directory.");
        //    d += "</title></head><body>";
        //    d += pURL.path();
        //    d += " : ";
        //    d += i18n("No such file or directory.");
        //    d += "</body></html>\n";
        //    begin(pURL);
        //    write(d);
        //    end();
        //    return;
        //}
    }
    kdDebug() << "sendTaviaRun :" << pURL.url() << endl;
    if (part == 0) {
        run = new TaviaRun(pURL,pArgs,this,this,this->view(),false,false,true);
        connect(run,SIGNAL(appRun()),this,SLOT(slotStopRun()));
    } else {
        new TaviaRun(pURL,pArgs,this,part,this->view(),false,false,true);
    }
}


/*!
    \fn TaviaHTMLView::slotChildOpenURL(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaHTMLView::slotChildOpenURL(const KURL &url,const KParts::URLArgs &args)
{
    if (dynamic_cast<KHTMLPart *>(const_cast<QObject *>(sender())) != 0) {
        kdDebug() << "openURL(child frame) : " << url.url() << endl;
        slotOpenURL(url,args,dynamic_cast<KHTMLPart *>(const_cast<QObject *>(sender())));
    }
}

/*!
    \fn TaviaHTMLView::slotMimeType(int job, const QString type)
 */
void TaviaHTMLView::slotMimeType(int job, const QString type)
{
    /// \todo ɤ߹߻νɲä뤳ȡ
    if (job != jobnum) return;
    if (type.lower().startsWith("image")) {
        kdDebug() << "!!!IMAGE" << endl;
        //end();
        noCache = true;
        //   outputImage(fileIO->getURL(job));
        //job->kill();
        return;
    }
    openURL(fileIO->getURL(job));
    //delete job;
    //begin(fileIO->getURL(job));
    disconnect(fileIO,SIGNAL(sendMimeType(int, const QString)),
        this,SLOT(slotMimeType(int, const QString)));
    //connect(fileIO,SIGNAL(sendData(int, const QByteArray)),
    //    this,SLOT(slotData(int, const QByteArray)));
}


/*!
    \fn TaviaHTMLView::slotData(int job,const QByteArray data)
 */
void TaviaHTMLView::slotData(int job,const QByteArray data)
{
    if (data.size() == 0) {
        end();
        disconnect(fileIO,SIGNAL(sendData(int, const QByteArray)),
            this,SLOT(slotData(int, const QByteArray)));
        return;
    }
    write(data.data(),data.size());
}


/*!
    \fn TaviaHTMLView::slotCompleted()
    URLɤ߹ߤ˽λemit륹åȡ
 */
void TaviaHTMLView::slotCompleted()
{
    slotCompleted(false);
}
/*!
    \fn TaviaHTMLView::slotCompleted(bool p)
    URLɤ߹ߤ˽λemit륹åȡ
 */
void TaviaHTMLView::slotCompleted(bool p)
{
    // ʲʤǥ꡼ʤå
    //if (run != 0) {
    //    delete run;
    //    run = 0;
    //}
    run = 0;
    kdDebug() << "!!!PENDING ACTION; " << p << endl;
    completed = true;
    emit pageNameReady(getPageName());
    emit pageURLReady(this,url());
    emit completedUrl(this,url());
    if (!noHistoryUpdate) addHistory();
    noCache = false;
    noHistoryUpdate = false;
    reloadHistory = false;
    stopAction = false;
    reloadAction = true;
    emit sendActionStatus();
    // debug
    for(unsigned int i = 0;i < urlHistory.count();i ++) {
        kdDebug() << "HISTORYDUMP:: " << i << "---" << urlHistory.at(i)->url.url() << endl;
    }
    //
    // favicon
    QByteArray data;
    QDataStream str(data, IO_WriteOnly);
    str << url();
    dynamic_cast<KApplication*>(KGlobal::instance())->dcopClient()->send("kded", "favicons", "downloadHostIcon(KURL)", data);
}

/*!
    \fn TaviaHTMLView::createThumbnail()
    ͥ롣
    λthumbnailReadyʥemit롣
 */
void TaviaHTMLView::createThumbnail()
{
    QPixmap pix;
    pix.resize(440,360);
    pix.fill(QColor(245,245,245));
    QRect rc(1,1,438,358);
    // ʲظ奿֥ץν
    if (view()->parentWidget()->isHidden()) {
        view()->parentWidget()->resize(438,358);
        view()->layout();
    }
    QPainter p;
    p.begin(&pix);
    paint(&p, rc);
    p.end();
    //view()->parentWidget()->updateGeometry();
    /* ꥵTSESSButtonǤƤߤ롣
    QImage img = pix.convertToImage();
    pix.convertFromImage(img.smoothScale(110,90));
    */
    thumbnailPixmap = pix;
    //emit thumbnailReady(thumbnailPixmap);
    kdDebug() << "Thumbnail Update." << endl;
    //if ((!completed) && (thumbnailUpdateTimer != 0))
    //    QTimer::singleShot(thumbnailUpdateTimer,this,SLOT(createThumbnail()));
}


/*!
    \fn TaviaHTMLView::aboutTavia()
    \todo ˥᥽åɤȤƼ٤ǤϤʤ
 */
void TaviaHTMLView::aboutTavia()
{
    noCache = true;
    QString d;
    d = "<html>\n";
    d += "<head><title>";
    d += KGlobal::instance()->aboutData()->programName();
    d += "</title></head>\n";
    d += "<body>\n";
    d += "<h1 style=\"text-align:center;\">";
    d += i18n("Tavia is a cheap web browser which aims too high.");
    d += "</h1>\n";
    d += "<div style=\"text-align:center;\"><img src=\"file:/";
    d += locate("data","tavia/tavialogo.png");
    d += "\" alt=\"TAVIALOGO\"/></div>\n";
    d += "<div style=\"font-weight:bold;font-size:200%;text-align:center;\">\n";
    d += i18n("Please type URL into URLBar,and push ENTER key.");
    d += "</div>\n";
    d += "<ul>\n";
    d += "<li style=\"font-weight:bold;\"><a href=\"http://nop.net-p.org/modules/pukiwiki/index.php?Tavia\">\n";
    d += i18n("Tavia Developers Web Page (described in Japanese)");
    d += "</a></li>\n";
    d += "<li style=\"font-weight:bold;\"><a href=\"http://www.kde.org/\">\n";
    d += i18n("K Desktop Environment");
    d += "</a></li>\n";
    d += "<li style=\"font-weight:bold;\"><a href=\"http://nop.net-p.org/modules/weblog/index.php?user_id=1\">\n";
    d += i18n("TAM's Web Diary (described in Japanese)");
    d += "</a></li>\n";
    d += "</ul>\n";
    d += "</html></body>\n";

    begin(KURL("about:tavia"));
    write(d);
    end();
}


/*!
    \fn TaviaHTMLView::aboutGPL()
    \todo ˥᥽åɤȤƼ٤ǤϤʤ
 */
void TaviaHTMLView::aboutGPL()
{
    noCache = true;
    begin(KURL("about:gpl"));
    write("<html><body><pre>");
    write(KGlobal::instance()->aboutData()->license());
    write("</pre></body></html>\n");
    end();
}

/*!
    \fn TaviaHTMLView::addHistory()
    ɲä롣
    \p reloadHistory trueξϥɤȽǤιΤ߹Ԥ
    \p noCache trueξsaveStateϹԤʤ˺ǿΥǡɤߤ˹Ԥ
    \todo QByteArray::resize(0)ɬåɤ٤
 */
void TaviaHTMLView::addHistory()
{
    TaviaHTMLCacheEntry *h;
    if (!reloadHistory) {
        //for(unsigned int i = currentHistory;i < urlHistory.count();i ++) {
        while(currentHistory < urlHistory.count()) {
            //delete urlHistory.last();
            // 줬ɬפʤΤפʤΤɬפʵ⤷ʤǤ̵ɡĤ䡢פ⡣
            //urlHistory.at(urlHistory.count() - 1)->state.resize(0);
            urlHistory.remove(urlHistory.count() - 1);
        }
        h = new TaviaHTMLCacheEntry;
        urlHistory.append(h);
        currentHistory = urlHistory.count();
    } else {
        h = urlHistory.at(currentHistory - 1);
    }
    if (!noCache) {
        QDataStream s(h->state,IO_WriteOnly);
        KHTMLPart::saveState(s);
        kdDebug() << "!!!saveState :" << h->state.size() << endl;
    }
    h->url = url();
    h->args = browserExtension()->urlArgs();
    h->noCache = noCache;
    h->title = getPageName();
    kdDebug() << "addHistory! (history count: " << urlHistory.count() << " NOCACHE:" << noCache << ")" << endl;
    createThumbnail();

    urlHistory.at(currentHistory - 1)->thumbnail = thumbnailPixmap;
    emit thumbnailReady(urlHistory.at(currentHistory - 1)->thumbnail);

    emit historyUpdate();
}

/*!
    \fn TaviaHTMLView::restoreHistory(unsigned int num)
    ꥹȥ롣סֿʤפФ륢ǻѡ
 */
void TaviaHTMLView::restoreHistory(unsigned int num)
{
    noHistoryUpdate = true;
    if (num > urlHistory.count()) return;
    // Historyǡ̵ɤ߹ߡ
    // ԴǤ롣
    currentHistory = num;
    if (urlHistory.at(num - 1)->noCache || (urlHistory.at(num - 1)->state.size() == 0)) {
        //reloadHistory = true;
        //slotOpenURL(urlHistory.at(num - 1)->url,urlHistory.at(num - 1)->args);
        emit reload();
        return;
    } else {
        QDataStream s(urlHistory.at(num - 1)->state, IO_ReadOnly );
        KHTMLPart::restoreState(s);
    }
    emit pageURLReady(this,urlHistory.at(num - 1)->url);
    emit thumbnailReady(urlHistory.at(num - 1)->thumbnail);
    emit historyUpdate();
}

/*!
    \fn TaviaHTMLView::getHistoryCount()
    ο֤
 */
unsigned int TaviaHTMLView::getHistoryCount()
{
    return(urlHistory.count());
}

/*!
    \fn TaviaHTMLView::getCurrentHistory()
    ߤΰ֤֤\p history Ф륤ǥint᤹
 */
unsigned int TaviaHTMLView::getCurrentHistory()
{
    return(currentHistory);
}

/*!
    \fn TaviaHTMLView::reload()
    ɤ롣̵뤷ɤ߹ߤԤ
 */
void TaviaHTMLView::reload()
{
    reloadHistory = true;
    emit slotOpenURL(urlHistory.at(currentHistory - 1)->url,urlHistory.at(currentHistory - 1)->args);
}

/*!
    \fn TaviaHTMLView::tSetURLArgs(const KURL &url, const KParts::URLArgs &args = KParts::URLArgs())
 */
void TaviaHTMLView::tSetURLArgs(const KURL &url, const KParts::URLArgs &args)
{
    KParts::URLArgs newargs = args;
    //newargs.metaData().erase("UserAgent");
    newargs.metaData()["UserAgent"] = "tavia-test";
    browserExtension()->setURLArgs(newargs);
}

/*!
    \fn TaviaHTMLView::slotStopRun()
    ץꥱưΤߡ
 */
void TaviaHTMLView::slotStopRun()
{
    run = 0;
    slotStop();
}

/*!
    \fn TaviaHTMLView::slotStop()
    ɤ߹ߤǤ롣
 */
void TaviaHTMLView::slotStop()
{
    closeURL();
    stopAction = false;
    reloadAction = true;
    emit sendActionStatus();
    if (run != 0) {
        run->abort();
    }
}

/*!
    \fn TaviaHTMLView::slotBack()
    ᤹
 */
void TaviaHTMLView::slotBack()
{
    slotBack(0);
}

/*!
    \fn TaviaHTMLView::slotForward()
    Ŀʤ롣
 */
void TaviaHTMLView::slotForward()
{
    slotForward(0);
}

/*!
    \fn TaviaHTMLView::slotBack(int num)
    +1᤹
 */
void TaviaHTMLView::slotBack(int num)
{
    restoreHistory(getCurrentHistory() - 1 - num);
}

/*!
    \fn TaviaHTMLView::slotForward(int num)
    +1ʤ롣
 */
void TaviaHTMLView::slotForward(int num)
{
    restoreHistory(getCurrentHistory() + 1 + num);
}

/*!
    \fn TaviaHTMLView::slotReload()
    ɤ߹ߥåȡ\ref reload() Ƥ֡
 */
void TaviaHTMLView::slotReload()
{
    emit reload();
    stopAction = false;
    emit sendActionStatus();
}

/*!
    \fn TaviaHTMLView::historyUpdate()
    \p history νԤ
 */
void TaviaHTMLView::historyUpdate()
{
    if (getCurrentHistory() <= 1) {
        backAction = false;
    } else {
        backAction = true;
    }
    if (getCurrentHistory() < getHistoryCount()) {
        forwardAction = true;
    } else {
        forwardAction = false;
    }
    emit sendActionStatus();
}

/*!
    \fn TaviaHTMLView::sendActionStatus(const KURL &url)
    Ƽܥξ֤ؼ롣ֿʤסΥܥ󤬲뤫ʤ
    äƥڡURLξŬڤ˻ؼ롣
 */
void TaviaHTMLView::sendActionStatus(const KURL &url)
{
    if (locked) {
        emit backActionEnabled(this,false);
        emit forwardActionEnabled(this,false);
        emit reloadActionEnabled(this,false);
    } else {
        emit backActionEnabled(this,backAction);
        emit forwardActionEnabled(this,forwardAction);
        emit reloadActionEnabled(this,reloadAction);
    }
    emit stopActionEnabled(this,stopAction);
    emit viewLockEnabled(this,locked);
    if (url.url().isNull()) {
        emit pageURLReady(this,TaviaHTMLView::url());
    } else {
        emit pageURLReady(this,url);
    }
}

/*!
    \fn TaviaHTMLView::slotSetStatusBarText(const QString& text)
    statusbarؤʸ롣
 */
void TaviaHTMLView::slotSetStatusBarText(const QString& text)
{
    emit sendStatusBarText(this,text);
}

/*!
    \fn TaviaHTMLView::outputErrorMessage(KIO::Job *job,KHTMLPart *part)
    HTMLȤƥ顼åϤ롣
 */
void TaviaHTMLView::outputErrorMessage(const KURL &url,KIO::Job *job,KHTMLPart *part)
{
    KURL pURL = url;
    noCache = true;
    QString d,opm;
    QString errorString;
    if (job == 0) {
        if (pURL.isLocalFile()) {
            errorString = i18n("No such file or directory.");
            opm = i18n("ERR_DOES_NOT_EXIST");
        } else {
            errorString = i18n("unknown error.");
        }
    } else {
        errorString = job->errorString();
        switch(job->error()){
        case KIO::ERR_CANNOT_OPEN_FOR_READING :
            opm = i18n("ERR_CANNOT_OPEN_FOR_READING");
            break;
        case KIO::ERR_INTERNAL :
            opm = i18n("ERR_INTERNAL");
            break;
        case KIO::ERR_MALFORMED_URL :
            opm = i18n("ERR_MALFORMED_URL");
            break;
        case KIO::ERR_UNSUPPORTED_PROTOCOL :
            opm = i18n("ERR_UNSUPPORTED_PROTOCOL");
            break;
        case KIO::ERR_NO_SOURCE_PROTOCOL :
            opm = i18n("ERR_NO_SOURCE_PROTOCOL");
            break;
        case KIO::ERR_UNSUPPORTED_ACTION :
            opm = i18n("ERR_UNSUPPORTED_ACTION");
            break;
        case KIO::ERR_DOES_NOT_EXIST :
            opm = i18n("ERR_DOES_NOT_EXIST");
            break;
        case KIO::ERR_UNKNOWN_HOST :
            opm = i18n("ERR_UNKNOWN_HOST");
            break;
        case KIO::ERR_ACCESS_DENIED :
            opm = i18n("ERR_ACCESS_DENIED");
            break;
        case KIO::ERR_CANNOT_ENTER_DIRECTORY :
            opm = i18n("ERR_CANNOT_ENTER_DIRECTORY");
            break;
        case KIO::ERR_PROTOCOL_IS_NOT_A_FILESYSTEM :
            opm = i18n("ERR_PROTOCOL_IS_NOT_A_FILESYSTEM");
            break;
        case KIO::ERR_CYCLIC_LINK :
            opm = i18n("ERR_CYCLIC_LINK");
            break;
        case KIO::ERR_COULD_NOT_CONNECT :
            opm = i18n("ERR_COULD_NOT_CONNECT");
            break;
        case KIO::ERR_NOT_FILTER_PROTOCOL :
            opm = i18n("ERR_NOT_FILTER_PROTOCOL");
            break;
        case KIO::ERR_COULD_NOT_READ :
            opm = i18n("ERR_COULD_NOT_READ");
            break;
        case KIO::ERR_COULD_NOT_LOGIN :
            opm = i18n("ERR_COULD_NOT_LOGIN");
            break;
        case KIO::ERR_SLAVE_DIED :
            opm = i18n("ERR_SLAVE_DIED");
            break;
        case KIO::ERR_OUT_OF_MEMORY :
            opm = i18n("ERR_OUT_OF_MEMORY");
            break;
        case KIO::ERR_UNKNOWN_PROXY_HOST :
            opm = i18n("ERR_UNKNOWN_PROXY_HOST");
            break;
        case KIO::ERR_COULD_NOT_AUTHENTICATE :
            opm = i18n("ERR_COULD_NOT_AUTHENTICATE");
            break;
        case KIO::ERR_ABORTED :
            opm = i18n("ERR_ABORTED");
            break;
        case KIO::ERR_UNKNOWN_INTERRUPT :
            opm = i18n("ERR_UNKNOWN_INTERRUPT");
            break;
        case KIO::ERR_SERVER_TIMEOUT :
            opm = i18n("ERR_SERVER_TIMEOUT");
            break;
        case KIO::ERR_NEED_PASSWD :
            opm = i18n("ERR_NEED_PASSWD");
            break;
        case KIO::ERR_UPGRADE_REQUIRED :
            opm = i18n("ERR_UPGRADE_REQUIRED");
            break;
        case KIO::ERR_POST_DENIED :
            opm = i18n("ERR_POST_DENIED");
            break;

        default:
            opm = "ERR_" + QString::number(job->error());
            break;
        }
    }
    d = "<html>\n<head>\n<title>";
    d += errorString;
    d += " - ";
    d += pURL.url();
    d += "</title>\n";
    d += "</head>\n<body>";
    d += "<table style=\"border:0px;padding:5px;margin:0px;border-style:solid;width: 100%;\" cellpadding=\"0\" border=\"0\">\n";
    d += "<tr><td style=\"width:200px;\">\n";
    d += "<img src=\"file:";
    d += locate("data","tavia/taviaInfo.png");;
    d += "\" alt=\"TAVIAINFO\" style=\"float:left\"/>\n";
    d += "</td><td style=\"padding:5px;\">\n";
    d += "<h1 style=\"color:red\">";
    d += errorString;
    d += "</h1>\n";
    d += opm;
    d += "</td></tr></table>\n";
    d += "</body>\n</html>\n";
    part->begin("taviainfo:" + pURL.url());
    //begin();
    part->write(d);
    part->end();
}

/*!
    \fn TaviaHTMLView::outputImage(const KURL &url)
    HTMLȤƥ᡼Ϥ롣
    \todo ΤȾüɬפǤ뤫ɤۣ档
 */
void TaviaHTMLView::outputImage(const KURL &url,KHTMLPart *part)
{
    KURL pURL = url;
    noCache = true;
    QString d;
    d = "<html>\n<head>\n<title>";
    d += i18n("Image");
    d += ":";
    d += url.url();
    d += "</title>\n</head>\n<body>\n<img src=\"";
    d += url.url();
    d += "\" />\n</body>\n</html>\n";
    part->begin(pURL.url());
    part->write(d);
    part->end();
}

/*!
    \fn TaviaHTMLView::popupMenu(const QString &linkurl)
    ݥåץåץ˥򵯤TaviaHTMLPopup 򻲾Ȥ٤
 */
void TaviaHTMLView::popupMenu(const QString &linkurl)
{
    kdDebug() << "popupMenu: " << linkurl << endl;

    // from Konqueror
    KURL popupURL;
    KURL linkKURL;
    if ( linkurl.isEmpty() )
        popupURL = this->url();
    else {
        popupURL = completeURL( linkurl );
        linkKURL = popupURL;
    }
    emit popup->menuPopup(popupURL,linkKURL);
}

/*!
    \fn TaviaHTMLView::partAdded (KParts::Part *part)
    ҥѡȤɲä줿˸ƤФ롣ե졼бǤ롣
 */
void TaviaHTMLView::partAdded (KParts::Part *part)
{
    kdDebug() << "PARTADDED!" << endl;
    if (part->inherits("KHTMLPart")) {
        connect(static_cast<KHTMLPart *>(part),SIGNAL(popupMenu(const QString &,const QPoint &)),SLOT(popupMenu(const QString &)));
        connect(static_cast<KHTMLPart *>(part)->partManager(),SIGNAL(partAdded(KParts::Part *)),this,SLOT(partAdded(KParts::Part *)));
        connect(static_cast<KHTMLPart *>(part)->partManager(),SIGNAL(partRemoved(KParts::Part *)),this,SLOT(partRemoved(KParts::Part *)));
        disconnect(static_cast<KHTMLPart *>(part)->browserExtension(),SIGNAL(openURLRequestDelayed(const KURL &,const KParts::URLArgs &)));
        disconnect(static_cast<KHTMLPart *>(part)->browserExtension(),SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)));
        connect(static_cast<KHTMLPart *>(part)->browserExtension(),SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
            this,SLOT(slotChildOpenURL(const KURL &,const KParts::URLArgs &)));

        //connect(static_cast<KHTMLPart *>(part)->browserExtension(),SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
        //    this,SLOT(slotOpenURL(const KURL &,const KParts::URLArgs &)));
    }
}

/*!
    \fn TaviaHTMLView::partRemoved (KParts::Part *part)
    ҥѡȤ줿˸ƤФ롣ߤϻѤƤʤ
 */
void TaviaHTMLView::partRemoved (KParts::Part *part)
{
    kdDebug() << "PARTREMOVED!" << endl;
}

/*!
    \fn TaviaHTMLView::getPageName()
    ڡȥ롣
    ȥβԤϽ롣
 */
QString TaviaHTMLView::getPageName()
{
    return(htmlDocument().title().string().replace(QChar('\n')," "));
}

/*!
    \fn TaviaHTMLView::setSlaveConfig()
 */
void TaviaHTMLView::setSlaveConfig()
{
    //KIO::SlaveConfig::self()->setConfigData("http","","UserAgent","Tavia-Test");
}

/*!
    \fn TaviaHTMLView::isLocked()
 */
bool TaviaHTMLView::isLocked()
{
    return(locked);
}


/*!
    \fn TaviaHTMLView::setLock(bool l)
 */
void TaviaHTMLView::setLock(bool l)
{
    locked = l;
    emit sendActionStatus();
}




/*!
    \fn TaviaHTMLView::progress(int p)
 */
void TaviaHTMLView::progress(int p)
{
    if (p == 100) return;
    if (p <= pProgress) return;
    pProgress = p;
    kdDebug() << "progress:: " << p << endl;
    createThumbnail();
    //
    // вߥ!!
    //
    QPixmap pix = thumbnailPixmap;
    QPainter paint;
    QPen pen(Qt::red,2);
    QBrush b(Qt::red,Qt::Dense6Pattern);
    paint.begin(&pix);
    paint.setPen(pen);
    paint.drawEllipse(220,180,200,160);
    paint.drawArc(220,180,200,170,-180*16,180*16);
    paint.setBrush(b);
    paint.drawPie(220,180,200,160,(90 * 16) - ((360 * 16 * p) / 100),(360 * 16 * p) / 100);
    kdDebug() << "Pie: " << (360 * 16 * 100) / p << endl;
    paint.end();
    progressPixmap = pix;
    thumbnailReady(progressPixmap);
}



/*!
    \fn TaviaHTMLView::getHistoryList()
 */
const QPtrList<TaviaHTMLCacheEntry> TaviaHTMLView::getHistoryList()
{
    return(urlHistory);
}


/*!
    \fn TaviaHTMLView::saveSession(KConfig *c)
 */
void TaviaHTMLView::saveSession(KConfig *c)
{
    c->writeEntry("Lock",isLocked());
    c->writeEntry("CurrentHistory",currentHistory);
    for(unsigned int i = 0;i < urlHistory.count();i ++) {
        QString entry;
        entry  = "URL_" + QString::number(i,10);
        c->writeEntry(entry,urlHistory.at(i)->url.url());
        entry  = "NoCache_" + QString::number(i,10);
        c->writeEntry(entry,urlHistory.at(i)->noCache);
    }
    c->sync();
}


/*!
    \fn TaviaHTMLView::loadSession(KConfig *c)
 */
void TaviaHTMLView::loadSession(KConfig *c)
{
    unsigned int i = 0;
    urlHistory.clear();
    while(-1) {
        QString n = "URL_" + QString::number(i,10);
        QString u = c->readEntry(n);
        if (u.isEmpty()) break;
        TaviaHTMLCacheEntry *e = new TaviaHTMLCacheEntry();
        e->url = KURL(u);
        n = "NoCache_" + QString::number(i,10);
        e->noCache = c->readBoolEntry(n,false);
        urlHistory.append(e);
        i ++;
    }
    currentHistory = c->readNumEntry("CurrentHistory",urlHistory.count());
    if (currentHistory >= i) currentHistory = i;
    if (urlHistory.count() > 0) {
        reload();
    }
    setLock(c->readBoolEntry("Lock",false));
}


/*!
    \fn TaviaHTMLView::getFavicon()
 */
const QPixmap TaviaHTMLView::getFavicon()
{
    KURL u;
    if (url().url().at(0) == '/') u.setPath(url().url());
    else u = url().url();
    QString icon = KMimeType::iconForURL(u);
    kdDebug() << "favicon : " << icon << endl;
    QPixmap x;
    return(x);

}





/*!
    \fn TaviaHTMLView::showError(KIO::Job *job)
 */
void TaviaHTMLView::showError(KIO::Job *job)
{
    if (job == 0) {
        outputErrorMessage(static_cast<KIO::TransferJob *>(job)->url(),0,this);
        return;
    }
    if (job->error()) outputErrorMessage(static_cast<KIO::TransferJob *>(job)->url(),job,this);
}
