/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviatab.h"
#include "taviaqnavi.h"
#include <qlabel.h>
#include <kdebug.h>

TaviaTab::TaviaTab(QWidget *parent, const char *name)
 : TSESSView(parent,name), KXMLGUIClient(), KBookmarkOwner()
{
    currentGUI = 0;
    htmlList.setAutoDelete(true);
    undoCache.setAutoDelete(true);
    setXMLFile(locate("data","tavia/taviatabui.rc"),true,true);
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    new KAction(i18n("add NewTab"),"tab_new",CTRL+Key_N,this,SLOT(newTab()),actionCollection(),"newTab");
    new KAction(i18n("remove Tab"),"tab_remove",CTRL+Key_W,this,SLOT(rmTab()),actionCollection(),"rmTab");
    new KAction(i18n("next Tab"),"1downarrow",SHIFT+Key_PageDown,this,SLOT(nextTab()),actionCollection(),"nextTab");
    new KAction(i18n("prev Tab"),"1uparrow",SHIFT+Key_PageUp,this,SLOT(prevTab()),actionCollection(),"prevTab");
    new KAction(i18n("QuickNavi"),"",CTRL+Key_Space,this,SLOT(qnavi()),actionCollection(),"qnavi");
    viewLock = new KToggleAction(i18n("View Lock"),"lock",SHIFT+CTRL+Key_L,actionCollection(),"viewLock");
    connect(viewLock,SIGNAL(toggled(bool)),this,SLOT(setViewLock(bool)));
    backAction = new KAction(i18n("Back"),"back",ALT+Key_Left,this,SLOT(pageBack()),actionCollection(),"pageBack");
    forwardAction = new KAction(i18n("Forward"),"forward",ALT+Key_Right,this,SLOT(pageForward()),actionCollection(),"pageForward");
    reloadAction = new KAction(i18n("Reload"),"reload",Key_F5,this,SLOT(pageReload()),actionCollection(),"pageReload");
    stopAction = new KAction(i18n("Stop"),"stop",Key_Escape,this,SLOT(pageStop()),actionCollection(),"pageStop");
    undoRmTabAction = new KAction(i18n("Undo RemoveTab"),"undo",SHIFT+CTRL+Key_Z,this,SLOT(undoRmTab()),actionCollection(),"undoRmTab");
    undoRmTabAction->setEnabled(false);
    uCombo = new TaviaURLBar(this,"urlBar");
    urlBar = new KWidgetAction(uCombo,i18n("url"),0,0,0,actionCollection(),"urlBar");
    urlFocus = new KAction(i18n("set focus to URLBar"),ALT+Key_O,uCombo,SLOT(setFocus()),actionCollection(),"urlBarSetForcus");
    urlBar->setAutoSized(true);
    connect(uCombo,SIGNAL(enterURL(const KURL &)),this,SLOT(slotOpenURL(const KURL &)));
    connect(this,SIGNAL(changed(int)),this,SLOT(guiSetting(int)));
    connect(tSessBBox,SIGNAL(rightMiniButtonClicked(int)),this,SLOT(delView(int)));
    connect(tSessBBox,SIGNAL(extWheelActionUp()),this,SLOT(prevTab()));
    connect(tSessBBox,SIGNAL(extWheelActionDown()),this,SLOT(nextTab()));
    connect(tSessBBox,SIGNAL(signalButtonShift(int, int )),this,SLOT(tabShift(int, int )));
    setBBoxShow(true);

    bookmarkManager = KBookmarkManager::managerForFile(locateLocal("data","tavia/bookmarks.xml"));
    bookmarkManager->setShowNSBookmarks(false);
    bookmarkMenuAction = new KActionMenu(i18n("&Bookmarks"),"bookmark",actionCollection(),"bookmarks");
    bookmarkMenu = new KBookmarkMenu(bookmarkManager,this,bookmarkMenuAction->popupMenu(),actionCollection(),true);

    tSessBBox->setFreeScale(true);


    QGridLayout *gray = new QGridLayout(dummyFrame,1,1);
    QLabel *logo = new QLabel(dummyFrame);
    logo->setPixmap(QPixmap(locate("data","tavia/taviabg.png")));
    logo->setScaledContents(false);
    logo->setAlignment(Qt::AlignLeft|Qt::AlignBottom);
    logo->setMinimumSize(10,10);
    gray->addWidget(logo,0,0);

    uCombo->setEnabled(false);
    urlFocus->setEnabled(false);
    backAction->setEnabled(false);
    forwardAction->setEnabled(false);
    reloadAction->setEnabled(false);
    stopAction->setEnabled(false);
    viewLock->setEnabled(false);

    //TaviaQNavi *qnavi = new TaviaQNavi(0,"NATA",Qt::WX11BypassWM | Qt::WShowModal);
}


TaviaTab::~TaviaTab()
{
    // ºݤDeleteˤ³ڤäƤ
    // פǤ롣ڤʤȣƤФƥեˤʤ뤾
    for(unsigned int i = 0;i < htmlList.count();i ++) {
        disconnect(getView(i),SIGNAL(destroyed(QObject*)),this,SLOT(destroyedView(QObject*)));
    }

    saveSession("taviasessionrc_last");
}

/*!
    \fn TaviaTab::delView(int num)
 */
void TaviaTab::delView(int num)
{
    // htmlList.autoDelete()Flseξ
    // HTMLViewDestroyǤȷᤦޥ
    if (getView(num)->isLocked() && htmlList.autoDelete()) {
        // ᥽åɲ٤Ȼפ졣
        getView(num)->setLock(false);
        tSessBBox->getTSESSButton(num)->tRightButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("fileclose",KIcon::Small,KIcon::SizeSmall));
        return;
    }

    // ºݤDeleteˤ³ڤäƤ
    // פǤ롣ڤʤȣƤФƥեˤʤ뤾
    disconnect(getView(num),SIGNAL(destroyed(QObject*)),this,SLOT(destroyedView(QObject*)));

    if (currentGUI == getView(num)) {
        factory()->removeClient(getView(num));
        currentGUI = 0;
    }
    // ٱ䥭å
    //  ΥɤϤޤƯʤ
    /*
    if (htmlList.autoDelete()) {
        KConfig *c = KGlobal::config();
        c->setGroup("TabUndo");
        unsigned int maxCache = c->readNumEntry("maxUndoCount",0);
        if (maxCache > 0) {
            if (undoCache.count() >= maxCache) {
                undoCache.remove((uint)0);
            }
            undoCache.append(getView(num));
            undoRmTabAction->setEnabled(true);
            htmlList.setAutoDelete(false);
            htmlList.remove((unsigned int)num);
            htmlList.setAutoDelete(true);
        } else {
            htmlList.remove((unsigned int)num);
        }
    } else {
        htmlList.remove((unsigned int)num);
    }
    */
    htmlList.remove((unsigned int)num);
    delFrame(num);
    if (count() == 0) {
    // 
        uCombo->setEnabled(false);
        urlFocus->setEnabled(false);
        backAction->setEnabled(false);
        forwardAction->setEnabled(false);
        reloadAction->setEnabled(false);
        stopAction->setEnabled(false);
        viewLock->setEnabled(false);
    }
    kdDebug() << "UndoRmTabCacheCount: " << undoCache.count() << endl;
    kdDebug() << "htmlListCount: " << htmlList.count() << endl;
}

/*!
    \fn TaviaTab::addView(const KURL &url)
 */
int TaviaTab::addView(const KURL &url)
{
    return(addView(url,KParts::URLArgs()));
}
/*!
    \fn TaviaTab::addView(const KURL &url,const KParts::URLArgs &args)
 */
int TaviaTab::addView(const KURL &url,const KParts::URLArgs &args)
{
    KParts::ReadOnlyPart *p = 0;
    return(addView(url,args,KParts::WindowArgs(),p));
}
/*!
    \fn TaviaTab::addView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p)
 */
int TaviaTab::addView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p)
{
    KURL pURL = url;
    KParts::URLArgs pArgs = args;
    if (count() == 0) {
    // 
        uCombo->setEnabled(true);
        urlFocus->setEnabled(true);
        viewLock->setEnabled(true);
    }
    int num = addFrame();
    // Ĥܥ󥢥
    tSessBBox->getTSESSButton(num)->tRightButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("fileclose",KIcon::Small,KIcon::SizeSmall));

    QVBoxLayout *l = new QVBoxLayout(getFrame(num));
    TaviaHTMLView *t = new TaviaHTMLView(pURL,pArgs,getFrame(num),"view");
    t->view()->show();
    t->view()->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    l->addWidget(t->view());
    htmlList.append(t);
    connect(t,SIGNAL(thumbnailReady(const QPixmap & )),
        tSessBBox->getTSESSButton(num),SLOT(setIcon(const QPixmap & )));
    connect(t,SIGNAL(pageNameReady(const QString &)),
        tSessBBox->getTSESSButton(num),SLOT(setLabel(const QString &)));
    connect(t,SIGNAL(backActionEnabled(TaviaHTMLView *,bool)),
        this,SLOT(backActionEnabled(TaviaHTMLView *,bool)));
    connect(t,SIGNAL(forwardActionEnabled(TaviaHTMLView *,bool)),
        this,SLOT(forwardActionEnabled(TaviaHTMLView *,bool)));
    connect(t,SIGNAL(reloadActionEnabled(TaviaHTMLView *,bool)),
        this,SLOT(reloadActionEnabled(TaviaHTMLView *,bool)));
    connect(t,SIGNAL(stopActionEnabled(TaviaHTMLView *,bool)),
        this,SLOT(stopActionEnabled(TaviaHTMLView *,bool)));
    connect(t,SIGNAL(viewLockEnabled(TaviaHTMLView *,bool)),
        this,SLOT(viewLockEnabled(TaviaHTMLView *,bool)));
    connect(this,SIGNAL(changed(int)),t,SLOT(sendActionStatus()));
    connect(t->browserExtension(),SIGNAL(createNewWindow(const KURL &,const KParts::URLArgs &)),
        this,SLOT(addView(const KURL &,const KParts::URLArgs &)));
    connect(t->browserExtension(),SIGNAL(createNewWindow(const KURL &,const KParts::URLArgs &,const KParts::WindowArgs &,KParts::ReadOnlyPart *&)),
        this,SLOT(addView(const KURL &,const KParts::URLArgs &,const KParts::WindowArgs &,KParts::ReadOnlyPart *&)));
    connect(this,SIGNAL(dropUrlList(int,const KURL::List)),
        this,SLOT(dropURLList(int,const KURL::List)));
    connect(t,SIGNAL(pageURLReady(TaviaHTMLView *,const KURL &)),
        this,SLOT(updateURLBar(TaviaHTMLView *,const KURL &)));
    connect(t,SIGNAL(completedUrl(TaviaHTMLView *,const KURL &)),
        this,SLOT(addURLBarHistory(TaviaHTMLView *,const KURL &)));
    connect(t,SIGNAL(sendStatusBarText(TaviaHTMLView *,const QString&)),
        this,SLOT(setStatusBarText(TaviaHTMLView *,const QString&)));
    connect(t,SIGNAL(destroyed(QObject*)),this,SLOT(destroyedView(QObject*)));
    setSelected(num);
    kdDebug()  << "!!! openURL : " << pURL.url() << endl;
    //getView(num)->slotOpenURL(url,args);
    //emit getView(num)->browserExtension()->openURLRequest(url,args);
    emit changed(num);
    //if (p != 0) p = getView(num);
    //p = getView(num);
    p = t;
    emit uCombo->setFocus();
    return(num);
}

/*!
    \fn TaviaTab::addView()
 */
int TaviaTab::addView()
{
    return(addView(KURL(),KParts::URLArgs()));
}

/*!
    \fn TaviaTab::getView(int num)
 */
TaviaHTMLView * TaviaTab::getView(int num)
{
    return(htmlList.at((unsigned int)num));
}

/*!
    \fn TaviaTab::newTab()
 */
void TaviaTab::newTab()
{
    addView();
}

/*!
    \fn TaviaTab::rmTab()
 */
void TaviaTab::rmTab()
{
    if (htmlList.count()) delView(selected);
}

/*!
    \fn TaviaTab::prevTab()
 */
void TaviaTab::prevTab()
{
    if (selected == 0) setSelected(count() - 1);
    else setSelected(selected - 1);
}

/*!
    \fn TaviaTab::nextTab()
 */
void TaviaTab::nextTab()
{
    if (selected == (count() - 1)) setSelected(0);
    else setSelected(selected + 1);
}

/*!
    \fn TaviaTab::pageBack()
 */
void TaviaTab::pageBack()
{
    emit getView(selected)->slotBack();
}

/*!
    \fn TaviaTab::pageForward()
 */
void TaviaTab::pageForward()
{
    emit getView(selected)->slotForward();
}

/*!
    \fn TaviaTab::pageReload()
 */
void TaviaTab::pageReload()
{
    emit getView(selected)->slotReload();
}

/*!
    \fn TaviaTab::pageStop()
 */
void TaviaTab::pageStop()
{
    emit getView(selected)->slotStop();
}

/*!
    \fn TaviaTab::backActionEnabled(TaviaHTMLView *t,bool b)
 */
void TaviaTab::backActionEnabled(TaviaHTMLView *t,bool b)
{
    if (isCurrentView(t)) backAction->setEnabled(b);
}

/*!
    \fn TaviaTab::forwardActionEnabled(TaviaHTMLView *t,bool b)
 */
void TaviaTab::forwardActionEnabled(TaviaHTMLView *t,bool b)
{
    if (isCurrentView(t)) forwardAction->setEnabled(b);
}

/*!
    \fn TaviaTab::reloadActionEnabled(TaviaHTMLView *t,bool b)
 */
void TaviaTab::reloadActionEnabled(TaviaHTMLView *t,bool b)
{
    if (isCurrentView(t)) reloadAction->setEnabled(b);
}

/*!
    \fn TaviaTab::stopActionEnabled(TaviaHTMLView *t,bool b)
 */
void TaviaTab::stopActionEnabled(TaviaHTMLView *t,bool b)
{
    if (isCurrentView(t)) stopAction->setEnabled(b);
}

/*!
    \fn TaviaTab::viewLockEnabled(TaviaHTMLView *t,bool b)
 */
void TaviaTab::viewLockEnabled(TaviaHTMLView *t,bool b)
{
    if (isCurrentView(t)) {
        viewLock->setChecked(b);
        if (b) {
            tSessBBox->getTSESSButton(selected)->tRightButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("lock",KIcon::Small,KIcon::SizeSmall));
        } else {
            tSessBBox->getTSESSButton(selected)->tRightButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("fileclose",KIcon::Small,KIcon::SizeSmall));
        }
    }
}

/*!
    \fn TaviaTab::isCurrentView(TaviaHTMLView *t)
 */
bool TaviaTab::isCurrentView(TaviaHTMLView *t)
{
    if (getView(selected) == t) return(true);
    else return(false);
}

/*!
    \fn TaviaTab::guiSetting(int num)
 */
void TaviaTab::guiSetting(int num)
{
    if (currentGUI == getView(num)) return;
    if (currentGUI != 0) factory()->removeClient(currentGUI);
    factory()->addClient(getView(num));
    currentGUI = getView(num);
}

/*!
    \fn TaviaTab::dropURLList(int num,const KURL::List list)
 */
void TaviaTab::dropURLList(int num,const KURL::List list)
{
    if (getView(num)->isLocked()) return;
    // ΤʣDropƤޤΤɤᡢŪDrop̵ˡ
    disconnect(this,SIGNAL(dropUrlList(int,const KURL::List)),
        this,SLOT(dropURLList(int,const KURL::List)));
    kdDebug() << "!!!DROP : " << list[0].url() << endl;
    getView(num)->browserExtension()->openURLRequest(list[0],KParts::URLArgs());
    for(unsigned int i = 1;i < list.count();i ++) {
        addView(list[i]);
    }
    connect(this,SIGNAL(dropUrlList(int,const KURL::List)),
        this,SLOT(dropURLList(int,const KURL::List)));
}

/*!
    \fn TaviaTab::updateURLBar(TaviaHTMLView *t,const KURL &url)
 */
void TaviaTab::updateURLBar(TaviaHTMLView *t,const KURL &url)
{
    if (t == getView(selected)) uCombo->setEditURL(url);
}

/*!
    \fn TaviaTab::addURLBarHistory(TaviaHTMLView *t,const KURL &url)
 */
void TaviaTab::addURLBarHistory(TaviaHTMLView *t,const KURL &url)
{
    uCombo->addURLHistory(url);
}


/*!
    \fn TaviaTab::setStatusBarText(TaviaHTMLView *t,const QString &text)
 */
void TaviaTab::setStatusBarText(TaviaHTMLView *t,const QString &text)
{
    if (t == getView(selected) || t == 0) {
        QString t = text;
        t.replace("<qt>",""); // ʤġ
        emit setStatusBarText(t);
    }
}

/*!
    \fn TaviaTab::slotOpenURL(const KURL &url)
 */
void TaviaTab::slotOpenURL(const KURL &url)
{
    emit slotOpenURL(url,KParts::URLArgs());
}

/*!
    \fn TaviaTab::slotOpenURL(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaTab::slotOpenURL(const KURL &url,const KParts::URLArgs &args)
{
    if (count() == 0) addView(url,args);
    else emit getView(selected)->browserExtension()->openURLRequest(url,args);
}


/*!
    \fn TaviaTab::openBookmarkURL(const QString &url)
 */
void TaviaTab::openBookmarkURL(const QString &url)
{
    if (htmlList.count() == 0) {
        addView(KURL(url),KParts::URLArgs());
    } else {
        getView(selected)->browserExtension()->openURLRequest(KURL(url),KParts::URLArgs());
    }
}

/*!
    \fn TaviaTab::currentTitle() const
 */
QString TaviaTab::currentTitle() const
{
    // ʤ󤫥ʤġ
    QPtrList<TaviaHTMLView> t = htmlList;
    int s = selected;
    return(t.at(s)->getPageName());
}

/*!
    \fn TaviaTab::currentURL() const
 */
QString TaviaTab::currentURL() const
{
    // ʤ󤫥ʤġ
    QPtrList<TaviaHTMLView> t = htmlList;
    int s = selected;
    return(t.at(s)->url().url());
}

/*!
    \fn TaviaTab::setViewLock(bool l)
 */
void TaviaTab::setViewLock(bool l)
{
/*
    if (l) {
        tSessBBox->getTSESSButton(selected)->tRightButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("lock",KIcon::Small,KIcon::SizeSmall));
    } else {
        tSessBBox->getTSESSButton(selected)->tRightButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("fileclose",KIcon::Small,KIcon::SizeSmall));
    }
*/
    getView(selected)->setLock(l);
}


/*!
    \fn TaviaTab::qnavi()
 */
void TaviaTab::qnavi()
{
    if (htmlList.count() == 0) return;
    navi = new TaviaQNavi(0,this,"NATA",Qt::WX11BypassWM | Qt::WShowModal);
    connect(navi,SIGNAL(qnaviClose()),this,SLOT(qnaviClose()));
}


/*!
    \fn TaviaTab::qnaviClose()
 */
void TaviaTab::qnaviClose()
{
    delete navi;
}



/*!
    \fn TaviaTab::destroyedView(QObject *t)
 */
void TaviaTab::destroyedView(QObject *t)
{
    htmlList.setAutoDelete(false);
    int c = htmlList.find(static_cast<TaviaHTMLView*>(t));
    currentGUI = 0;
    delView(c);
    htmlList.setAutoDelete(true);
}




/*!
    \fn TaviaTab::tabShift(int num, int shift)
 */
int TaviaTab::tabShift(int num, int shift)
{
    if (shift == num) return(num);
    TaviaHTMLView *c = getView(getSelected());
    TaviaHTMLView *t = getView(num);
    kdDebug() << "SHIFT " << num << " to " << shift << endl;
    htmlList.setAutoDelete(false);
    htmlList.replace(num,htmlList.at(shift));
    htmlList.replace(shift,t);
    htmlList.setAutoDelete(true);
    frameShift(num,shift);
    setSelected(htmlList.find(c));
    return(htmlList.find(t));
}


/*!
    \fn TaviaTab::saveSession(QString name)
 */
void TaviaTab::saveSession(QString name)
{
    KConfig *c = new KConfig(name);
    // ˾ä
    // äȴñ˾äʤɡġ
    QStringList l = c->groupList();
    for(unsigned int i = 0;i < l.count();i ++) {
        c->deleteGroup(l[i]);
    }
    c->sync();

    c->setGroup("Session");
    c->writeEntry("selected",getSelected());

    for(unsigned int i = 0;i < htmlList.count();i ++) {
        QString g = "TAB_" + QString::number(i,10);
        c->setGroup(g);
        htmlList.at(i)->saveSession(c);
    }
    c->sync();
    delete c;
}


/*!
    \fn TaviaTab::loadSession(QString name)
 */
void TaviaTab::loadSession(QString name)
{
    KConfig *c = new KConfig(name,true); // RO˿饤󥲡ͤ
    c->setGroup("Session");
    int s = c->readNumEntry("selected",-1);
    unsigned int hc = htmlList.count();
    unsigned int i = 0;
    while(-1) {
        QString g = "TAB_" + QString::number(i,10);
        if (!c->hasGroup(g)) break;
        int v = addView();
        c->setGroup(g);
        htmlList.at(v)->loadSession(c);
        i ++;
    }
    if ((s >= 0) && (htmlList.count() > (hc + s))) {
        setSelected(s + hc);
    }
    delete c;
}


/*!
    \fn TaviaTab::undoRmTab()
 */
void TaviaTab::undoRmTab()
{
    /// @todo implement me
}
